/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef _HARK__BEAMFORMING__GHDSS_H__INCLUDED_
#define _HARK__BEAMFORMING__GHDSS_H__INCLUDED_ 1

#include "BF.h"

namespace BF
{
    class GHDSS : public BF
    {
    public:
        GHDSS(int numFreq, int lowerfreq, int upperfreq, int samplingRate, double soundSpeed, bool muAdp, double mu, bool mu_ssAdp, double mu_ss, double scale, NlFuncType nlFuncType, UtUpdateType separationMatUpdateType, double noisefloor, LcConstType lcConstType, UsUpdateStepType updateStepType, bool useLcmyuAsSsmyu);
	virtual ~GHDSS();

    public:
	virtual BfType getBfAlgorithmType() const { return BfGHDSS; }

	virtual void updateSources(const std::vector<SourceLocation> &sourcePositions);
	virtual void adaptSeparationMatrices();

    private:
	NlFuncType nlFuncType;
	double mu, mu_ss, scale;
	bool muAdp, mu_ssAdp, useLcmyuAsSsmyu;
	
    /* (GHDSS) 入力信号をノイズとみなす振幅の閾値(上限)  */
    double noisefloor;
    
    /* (GHDSS) 幾何制約の手法 */
    LcConstType lcConstType;
    
    /* (GHDSS) 分離行列の更新方法 */
    UsUpdateStepType updateStepType;
    };
}

#endif
