/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef _HARK__BEAMFORMING__LCMV_H__INCLUDED_
#define _HARK__BEAMFORMING__LCMV_H__INCLUDED_ 1

#include "BF.h"

namespace BF
{
    class LCMV : public BF
    {
    public:
        LCMV(int numFreq, int lowerfreq, int upperfreq, int samplingRate, double soundSpeed, bool muAdp, double mu, UtUpdateType separationMatUpdateType);
	virtual ~LCMV();

    public:
	virtual BfType getBfAlgorithmType() const { return BfLCMV; }

	virtual void updateSources(const std::vector<SourceLocation> &sourcePositions);
	virtual void adaptSeparationMatrices();

    private:
	std::vector<Matrix> F; /* sizs_is(numFreq), element_is(Matrix(numSrc, numMic)) */
	std::vector<Matrix> P; /* sizs_is(numSrc * numFreq), element_is(Matrix(numMic, numMic)) */
	const double mu;
	bool muAdp;
    };
}

#endif
