/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

// ---------------------------------------------------------------------
#ifndef	CPF_exec_h_defined
#define	CPF_exec_h_defined
// ---------------------------------------------------------------------
#include "ArrayInit.h"
//#include "ArrayUtil.h"
#include "PF_exec.h"
#include "PF_Init.h"
#include "MICARY_PF_Struct.h"

#include <string>
#include <iostream>
#include <vector>
// ---------------------------------------------------------------------
/**
 \class CPF_exec
 \brief \~English Class for executing Post-Filter process
 \brief \~Japanese Post-Filter̎sNX
*/
class CPF_exec{
private:
  //MICARY_FloatSpecArray *m_pfsNoiseSpec ;	//wiGSpectra
	MICARY_SavedMCRA *m_psSavedMCRA ;	//MCRAۑl(current frame)
	MICARY_SavedEstRev *m_psSavedRev ;	//cۑl(current frame)
	MICARY_SavedEstSN *m_psSavedEstSN ;	//SN䐄ۑl
	MICARY_SavedVoiceP *m_psSavedVoiceP ;	//݊mۑl
	MICARY_SavedMCRA *m_psSavedMCRATmp ;	//MCRAۑlTmp(1Ot[)
	MICARY_FloatSpecArray *m_pfsSavedLeak ;	//Rۑl
	MICARY_SavedEstRev *m_psSavedRevTmp ;	//cۑlTmp(1Ot[)
	MICARY_FloatSpecArray *m_pfsLamda_total ; //Total noise

	// Temporary array for change source(|C^ɕύX@2008/2/15)
	MICARY_SavedMCRA *tmp_sSavedMCRA,*tmp_sSavedMCRATmp ;//MCRAۑl(temporary)
	MICARY_SavedEstRev *tmp_sSavedRev,*tmp_sSavedRevTmp ;//cۑl(temporary)
	MICARY_SavedEstSN *tmp_sSavedEstSN ;	//SN䐄ۑl(temporary)
	MICARY_SavedVoiceP *tmp_sSavedVoiceP ;	//݊mۑl(temporary)
	MICARY_FloatSpecArray *tmp_fsSavedLeak ;	//Rۑl(temporary)
	MICARY_FloatSpecArray *tmp_fsLamda_total ; // total lamda(temporary)
	
	
public:
	virtual int ChangeSrcPos(MICARY_FloatSpecArray faInStatNoise,MICARY_IntArray iaSrcCur,MICARY_IntArray iaSrcIni,HA_Float ha_fNoise_Compense);
	virtual int InitSavedArray(MICARY_FloatSpecArray faInStatNoise,int iLower_freq,int iUpper_freq,int iNpos,int iNch,HA_Float ha_fNoise_compense);
	/**
	 \brief \~Japanese RXgN^
	*/
	CPF_exec() ;
	/**
	 \brief \~Japanese fXgN^
	*/
	~CPF_exec() ;
	/**
	 \brief \~Japanese NA
	*/
	int Clear() ;
	MICARY_FloatSpecArray *m_pfsNoiseSpec ;	//wiGSpectra

	// PFpParameter\
	MICARY_PrmVoiceP sPrmVoiceP ;	//powerp[^
	MICARY_PrmEstSN sPrmEstSN ;	//S/N䐄p[^
	MICARY_PrmRevNoise sPrmRev ;	//cp[^
	MICARY_PrmLeakNoise sPrmLeak ;	//Rꐬp[^
	MICARY_PrmMCRA sPrmMCRA ;	//MCRAp[^	
	MICARY_PrmGain sPrmGain ;	//œKGainp[^

	/**
	 \brief \~Japanese Post-Filterpۑlz̊m
	 \param iLower_freq \~English Index of lower bound frequency
	 \~Japanese gCfbNX
	 \param iUpper_freq \~English Index of upper bound frequency
	 \~Japanese gCfbNX
	 \param iNpos \~English Size of sources
	 \~Japanese 
	 \param iNch \~English Size of mic channels
	 \~Japanese }CN`l
	 \param iNproc \~English Number of process
	 \~Japanese 
	*/
	int Initialize(int iLower_freq,int iUpper_freq,int iNch,int iNproc) ;
	/**
	 \brief \~Japanese Post-Filterpۑlz̏
	 \param faInStatNoise \~English Power spectra estimated the state noise of each channel
	 \~Japanese 肳ꂽMCRApmCỸp[XyNg
	 \param iLower_freq \~English Index of lower bound frequency
	 \~Japanese gCfbNX
	 \param iUpper_freq \~English Index of upper bound frequency
	 \~Japanese gCfbNX
	 \param iNpos \~English Size of sources
	 \~Japanese 
	 \param iNch \~English Size of mic channels
	 \~Japanese }CN`l
	*/
	int InitSavedArray(MICARY_FloatSpecArray faInStatNoise,int iLower_freq,int iUpper_freq,int iNpos,int iNch);
	/**
	 \brief \~Japanese MCARpp[^̐ݒ
	 \param fNoise_compens \~English Noise compens
	 \~Japanese G␳l
	 \param fSmooth_spec_factor \~English Smoothing factor for input spectra
	 \~Japanese ̓XyNg̕W
	 \param fAmp_leak_factor \~English Leak factor of amplitude
	 \~Japanese URW
	 \param fBacknoise_est_factor \~English Smoothing factor for estimated BGN
	 \~Japanese wiG̕W
	 \param fLeak_factor \~English Leak factor
	 \~Japanese RW
	 \param fSs_floor \~English Floor factor
	 \~Japanese floorW
	 \param iL \~English Time width for detection
	 \~Japanese oԕ
	 \param fDelta \~English Threshold level of power ratio
	 \~Japanese p[臒l
	*/
	int Set_PF_PrmMCRA(float fNoise_compens,float fSmooth_spec_factor,float fAmp_leak_factor,
		float fBacknoise_est_factor, float fLeak_factor,float fSs_floor,int iL,float fDelta) ;
	/**
	 \brief \~Japanese Rꐄpp[^̐ݒ
	 \param fLeak_factor \~English Leak factor
	 \~Japanese RW
	 \param fOver_cancel \~English Over cancel value
	 \~Japanese I[o[LZl
	*/
	int Set_PF_PrmLeak(float fLeak_factor,float fOver_cancel) ;
	/**
	 \brief \~Japanese cpp[^̐ݒ
	 \param fReverb_decay \~English Reverb decay value
	 \~Japanese c萔
	 \param fReverb_level \~English Direct component ratio
	 \~Japanese ڐ
	*/
	int Set_PF_PrmRev(float fReverb_decay,float fReverb_level) ;
	/**
	 \brief \~Japanese SN䐄pp[^̐ݒ
	 \param fPrior_factor \~English Time SN ratio
	 \~Japanese SN
	 \param fAlp_pmag \~English Amplitude factor of voice inclusion rate
	 \~Japanese ܗL̐UW
	 \param fAlp_pmin \~English Minimum voice inclusion rate
	 \~Japanese ŏܗL
	 \param fXi_max \~English Max value of posterior SN ratio
	 \~Japanese SN̍ől
	 \param fTheta_max \~English Max value of optimum gain mid variable(v)
	 \~Japanese œKQCԕϐ(v)̍ől
	 \param fTheta_min \~English Minimum value of optimum gain mid variable(v)
	 \~Japanese œKQCԕϐ(v)̍ŏl
	*/
	int Set_PF_PrmSN(float fPrior_factor,float fAlp_pmag,float fAlp_pmin,float fXi_max,
		float fTheta_max,float fTheta_min) ;
	/**
	 \brief \~Japanese mpp[^̐ݒ
	 \param fSmooth_prior_factor \~English Smoothing factor of prior SN ratio
	 \~Japanese OSN̕W
	 \param fZminF \~English Minimum value of smoothing SN ratio(frame)
	 \~Japanese SN̍ŏl(frame)
	 \param fZmaxF \~English Max value of smoothing SN ratio(frame)
	 \~Japanese SN̍ől(frame)
	 \param fZminG \~English Minimum value of smoothing SN ratio(global)
	 \~Japanese SN̍ŏl(global)
	 \param fZmaxG \~English Max value of smoothing SN ratio(global)
	 \~Japanese SN̍ől(global)
	 \param fZminL \~English Minimum value of smoothing SN ratio(local)
	 \~Japanese SN̍ŏl(local)
	 \param fZmaxL \~English Max value of smoothing SN ratio(local)
	 \~Japanese SN̍ől(local)
     \param fBwFen \~English Max value of smoothing interval(frame)
	 \~Japanese Ԃ̍ől(frame)
     \param fBwFst \~English Minimum value of smoothing interval(frame)
	 \~Japanese Ԃ̍ŏl(frame)
     \param fBwG \~English Width of smoothing interval(global)
	 \~Japanese ԕ(global)
     \param fBwL \~English Width of smoothing interval(local)
	 \~Japanese ԕ(local)
	 \param fZthF \~English Threshold of smoothing SN ratio(frame)
	 \~Japanese SN臒l(frame)
	 \param fZpmin \~English Minimum peak value of smoothing SN ratio
	 \~Japanese SNs[N̍ŏl
	 \param fZpmax \~English Max peak value of smoothing SN ratio
	 \~Japanese SNs[N̍ől
	 \param fAmpF \~English Smoothing factor of voice probability(frame)
	 \~Japanese mW(frame)
	 \param fAmpG \~English Smoothing factor of voice probability(global)
	 \~Japanese mW(global)
	 \param fAmpL \~English Smoothing factor of voice probability(local)
	 \~Japanese mW(local)
	 \param fQmin \~English Minimum value of break probability
	 \~Japanese x~m̍ŏl
	 \param fQmax \~English Max value of break probability
	 \~Japanese x~m̍ől
	*/
	int Set_PF_PrmVoiceP(float fSmooth_prior_factor,float fZminF,float fZmaxF,float fZminG,float fZmaxG,
		float fZminL,float fZmaxL,int iBwFen,int iBwFst,int iBwG,int iBwL,float fZthF,
		float fZpmin,float fZpmax,float fAmpF,float fAmpG,float fAmpL,float fQmin,float fQmax) ;
	/**
	 \brief \~Japanese ŏQCp[^̐ݒ
	 \param fMinGain \~English Minimum gain while without speeching
	 \~Japanese ̍ŏQC
	*/
	int Set_PF_PrmGain(float fMinGain) ;
	/**
	 \brief \~Japanese Post-Filter̎s
	 \param csInputCSpec \~English Input signal spectra(complex)
	 \~Japanese ͐MXyNg(complex)
	 \param pcsOutputCSpec \~English Output signal spectra(complex)
	 \~Japanese o͐MXyNg(complex)
	 \param pfsEstNoise \~English Estimated noise power spectra
	 \~Japanese Gp[XyNg
	*/
	int ExecutePF(MICARY_ComplexSpecArray csInputCSpec,MICARY_ComplexSpecArray *pcsOutputCSpec,
		MICARY_FloatSpecArray *pfsEstNoise) ;
	/**
	 \brief \~Japanese ύX̏
	 \param faInStatNoise \~English Power spectra estimated the state noise of each channel
	 \~Japanese 肳ꂽMCRApmCỸp[XyNg
	 \param iaSrcCur \~English Source ID array for current frame
	 \~Japanese t[̉IDz
	 \param iaSrcIni \~English Source ID array in front of frame
	 \~Japanese 1t[ỎIDz
	*/
	int ChangeSrcPos(MICARY_FloatSpecArray faInStatNoise,MICARY_IntArray iaSrcCur,MICARY_IntArray iaSrcIni) ;
		
};
// ---------------------------------------------------------------------
#endif // CPF_exec_h_defined
