/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

// ---------------------------------------------------------------------
#ifndef	CPF_init_state_noise_h_defined
#define	CPF_init_state_noise_h_defined
// ---------------------------------------------------------------------
#include "MICARYHA_struct.h"
#include "MICARY_struct.h"
#include "MICARY_PF_Struct.h"
#include "PF_values.h"
#include "ArrayInit.h"
// ---------------------------------------------------------------------
/**
 \class CPF_init_state_noise
 \brief \~English Class for estimating the state noise of the each channnel
 \brief \~Japanese MCRApmCYNX
*/
class CPF_init_state_noise{
private:
	MICARY_SavedInitMCRA	*m_psSavedInitMCRA;	// ̓`lMCRAOt[ۑl
	MICARY_PrmInitMCRA	m_sPrmInitMCRA;	// MCRAp[^
	
public:
	/**
	 \brief \~Japanese RXgN^
	*/
	CPF_init_state_noise();
	/**
	 \brief \~Japanese fXgN^
	*/
	~CPF_init_state_noise();
	
	/**
	 \brief \~Japanese NA
	*/
	int Clear();

	/**
	 \brief \~Japanese MCRApmCYNX̏
	 \param iLowerFreq \~English Lower frequency[line]
	 \~Japanese gC
	 \param iUpperFreq \~English Upper frequency[line]
	 \~Japanese gC
	 \param iNCh \~English Number of channel
	 \~Japanese `l
	*/
	int Initialize(int iLowerFreq, int iUpperFreq, int iNCh);


	/**
	 \brief \~Japanese MCRApmCY̎擾
	 \param faInputPSpec \~English Input power spectrum (N, M)
	 \~Japanese ̓p[XyNg
	 \param pfsInitNoisePSpec \~English MCRA values
	 \~Japanese MCRAmCY
	*/
	int InitStateNoise(MICARY_FloatSpecArray faInputPSpec,
                       MICARY_FloatSpecArray *pfsInitNoisePSpec);

	/**
	 \brief \~Japanese MCRApmCYpp[^̐ݒ
	 \param sPrmInitMCRA \~English Parameters for Initial noise of MCRA calc.
	 \~Japanese MCRAmCYvZp[^
	*/
	int SetPrmInitMCRA(MICARY_PrmInitMCRA sPrmInitMCRA);
};
// ---------------------------------------------------------------------
#endif // CPF_init_state_noise_h_defined
