/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef ___ESTIMATIONTF_HPP___
#define ___ESTIMATIONTF_HPP___

#include "BF_Matrix.h"

using namespace std;

class EstimationTransferFunction
{
public:
	EstimationTransferFunction(int sampling_rate, int fft_length, bool is_debug,  
	                           string dir_export, bool is_export, int history_frames, int history_period);
//	virtual ~EstimationTransferFunction();

	bool getBasefromFile(const string &file_name, int &num_frq, int &num_mic, int &num_src);
	bool getBaseinDirectory(const string &dir_name, int &num_frq, int &num_mic, int &num_src);

	void initHistoryData();

	bool exportIntermData(const string &dir_name, int i_row);
	bool exportHistoryData(const string &dir_name, int i_row);
	bool exportTransferFunction(const string &name, bool isDir);

	void copyKeepTF();

	void estimateTansferFunction(int count, BF::Matrix &x, vector<BF::SourceLocation> sources);

private:
	void initIntermData();

	BF::Matrix _RelaTF(BF::Matrix X);
	void _getIntermediate(int tfId, float &id, float &x, float &y, float &z, float &frames, 
                          BF::Matrix &myu, BF::Matrix &sig);
	void _setIntermediate(int tfId, float frames, BF::Matrix myu, BF::Matrix sig);

	int SamplingRate;
	bool IsDebug;
	string DirExport;
	bool IsExport;
	int HistoryFrames;
	int HistoryPeriod;
	int numFFT;
	int numFreq;
	int numMic;
	int numSrc;

	BF::TransformFunctionDatabase TransFuncDatabase;
	BF::FMatrix IntermData;
	BF::FMatrix HistoryData;
	
	int idxHist;
	
	vector<BF::Matrix> keepTFdata;
};

#endif //___ESTIMATIONTF_HPP___
