/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef	HarkFileIOByMatrix_NH_h_defined
#define	HarkFileIOByMatrix_NH_h_defined
// ---------------------------------------------------------------------
#include "BF_Matrix.h"

#include <vector>

#ifdef ENABLE_HARKIO3
#else
#include "HarkFileIO.h"
#endif

class CHarkFileIOByMatrix
{
public:
	CHarkFileIOByMatrix();
	~CHarkFileIOByMatrix();
	
	// tfファイルのオープン(ロード)(cFilenameが既に開いている場合はスルーする)
	bool OpenTFfile(const char* cFilename, const char* cFileType);

	// tfディレクトリのオープン(ロード)(cDirnameが既に開いている場合はスルーする)
	bool OpenTFfileFromDir(const char* cDirname, const char* cFileType);

	// tfファイルの再オープン
	bool ReopenTFfile(const char* cFilename, const char* cFileType);

	// tfディレクトリの再オープン
	bool ReopenTFfileFromDir(const char* cDirname, const char* cFileType);

	// tfファイルのクローズ(TFメンバの開放)
	bool CloseTFfile();
	
	// ファイル読み込み
	bool ReadHeader(const char* cFilename, const char* cFileType,
					long *plFs,
					long *plNFFT,
					long *plNSrc,
					long *plNMic);
	
	// TFデータを読込み
	// threshに負の値を入力したとき、TFファイルに登録されたneighborを読み出す(2020/03/16new!)
	bool ReadTFData(const char* cFilename, 
					float thresh,
					BF::TransformFunctionDatabase &sDatabase);

	// TFデータをディレクトリから読込み
	// threshに負の値を入力したとき、TFファイルに登録されたneighborを読み出す(2020/03/16new!)
	bool ReadTFDataFromDir(const char* cDirname, 
					float thresh,
					BF::TransformFunctionDatabase &sDatabase);

	// TFデータを書き出し(2020/03/16new!)
	bool WriteTFData(const char* cFilename, 
					long lFs,
					long lNFFT,
					const char* comment,
					BF::TransformFunctionDatabase sDatabase);

	// TFデータをディレクトリに書き出し(2020/03/16new!)
	bool WriteTFDataToDir(const char* cDirname, 
					long lFs,
					long lNFFT,
					const char* comment,
					BF::TransformFunctionDatabase sDatabase);

	// TFデータをディレクトリに書き出し(2020/03/16new!)
	// 指定した音源位置の伝達関数を更新
	bool UpdateTFDataToDir(const char* cDirname, 
					int isrcindex,
					BF::TransformFunctionDatabase sDatabase);

	// SPデータを読込み
	bool ReadWData(const char* cFilename, 
					BF::Database &sDatabase);

	// SPデータを書込み
	bool WriteWData(const char* cFilename, 
					long lFs,
					long lNFFT,
					BF::Database sDatabase);

	
	// メンバー
	char*	m_pcTFfilename;	///< TFファイル名
	char*	m_pcFileType;
#ifdef ENABLE_HARKIO3
	harkio_TransferFunction	*m_ptf;	///< TFファイルポインタ
#else
	CHarkFileIO	m_ctf;
#endif
	
private:
	void _CalcNeighbors(std::vector<BF::SourceLocation> sourceLocations, float thresh, std::vector<std::vector<int> > &viNbrs);

	// TFデータを読込み
	// threshに負の値を入力したとき、TFファイルに登録されたneighborを読み出す
	bool _ReadTFData(float thresh,
					BF::TransformFunctionDatabase &sDatabase);

	// TFデータを書き出し
	bool _WriteTFData(BF::TransformFunctionDatabase sDatabase,
					long lFs,
					long lNFFT,
					const char* comment,
					harkio_TransferFunction **tf);
};

// 以下すべて(2020/03/16new!)
#ifdef ENABLE_HARKIO3
class CHarkMatFileIOByFMatrix
{
public:
	CHarkMatFileIOByFMatrix();
	~CHarkMatFileIOByFMatrix();

	// matファイルのオープン(cFilenameが既に開いている場合はスルーする)
	bool OpenMatfile(const char* cFilename);

	// matファイルの再オープン
	bool ReopenMatfile(const char* cFilename);
	
	// matファイルのクローズ
	bool CloseMatfile();
	
	// Matデータを読込み
	bool ReadMatData(const char* cFilename, 
					BF::FMatrix &sMat);

	// Matデータを書き込み
	bool WriteMatData(const char* cFilename, 
					BF::FMatrix sMat);

	// Matデータを行書き換え
	bool ReplaceMatData(const char* cFilename, 
					BF::FMatrix sMat,
					int iRowindex);
	
	// メンバー
	char*	m_pcMatfilename;	///< matファイル名
	harkio_Matrix	*m_pmat;	///< matファイルポインタ
};
#endif

#endif
