/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef ___LOCALIZATIONBFDS_HPP___
#define ___LOCALIZATIONBFDS_HPP___

#include "BF_Matrix.h"

using namespace std;

class LocalizationBFDS {
public:
	LocalizationBFDS(int sampling_rate, int fft_length, float sound_speed);

	const vector<BF::SourceLocation>& GetSrcPositions() { return this->SrcPositions; }

	bool ReadTransferFunction(const string &file_name, vector<int> &ch_select, int &num_mic, int &num_src);

	bool ReadXMLmicPos(const char *file_name, vector<int> &ch_select, int &num_mic);
	bool ReadXMLsrcPos(const char *file_name, int &num_src);

	void SetBfDsFilterfromTF(vector<BF::Matrix> &bf_filter);
	void SetBfDsFilterfromPOS(vector<BF::Matrix> &bf_filter);

	void Localize(BF::Matrix &x, vector<BF::Matrix> &bf_filter, vector<float> &bf_power, int &found_id);

private:
	int SamplingRate;
	float SoundSpeed;
	int numFFT;
	int numFreq;
	int numMic;
	int numSrc;

	vector<BF::SourceLocation> SrcPositions;
	vector<BF::MicLocation>    MicPositions;
	vector<BF::Matrix> Adata; // size(numSrc), elemnent(numMic,numFreq)

	float MicXC, MicYC, MicZC;
};

#endif //___LOCALIZATIONBFDS_HPP___
