/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef ___LOCALIZATIONCSP_HPP___
#define ___LOCALIZATIONCSP_HPP___

#include "BF_Matrix.h"

#include "fft.hpp"

using namespace std;

class LocalizationCSP {
public:
	LocalizationCSP(int tlength, vector<float> mssw, 
	                int min_tau, int max_tau, float csp_threshold, int maxn_out_peaks);

	void AddCrossCorrelation(vector<BF::Complex> &rx, BF::Matrix &X);
	void NormalizeCrossCorrelation(vector<BF::Complex> &rx, int sum_count);
	void ClearCrossCorrelation(vector<BF::Complex> &rx);

	int Localize(vector<BF::Complex> &rx, vector<float> &csp, vector<int>& found_id, vector<float> &found_csp);

private:
	int TimeLength;
	int FreqLength;
	vector<float> Mssw;
	int MinTau;
	int MaxTau;
	float CspThreshold;
	int MaxNumOutputPeaks;

	rfft<float> rfft1;
//	vector<float> win_conj;

	int SearchLocation(vector<float> &csp, vector<int>& found_id, vector<float> &found_csp);
	void SearchPeak(vector<float> &csp, vector<int>& peak_cand_id, vector<float> &peak_cand_val);
};

#endif //___LOCALIZATIONCSP_HPP___
