/*
 * Copyright (c) 2004 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#ifndef	PF_EXEC_h_defined
#define	PF_EXEC_h_defined
// ---------------------------------------------------------------------
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "MICARYHA_struct.h"
#include "ArrayInit.h"
#include "MICARY_PF_Struct.h"
//#include "MICARY_SpecFile.h"
#include "PF_Init.h"
#include "PF_est_leak.h"
#include "PF_est_noise.h"
#include "PF_est_rev.h"
#include "PF_est_sn.h"
#include "PF_est_state_noise.h"
#include "PF_est_voice_p.h"

// 2007/07/13 added by S.Suzuki C++CĂяo
#ifdef __cplusplus
extern "C" {
#endif


// ---------------------------------------------------------------------

/**
 \brief \~Japanese PF̎s(1:I, 0:G[)
 \param csInputSpec \~English Input spectrum
 \~Japanese ̓XyNg
 \param pcsOnputSpec \~English Output spectrum 
 \~Japanese o̓XyNg
 \param pSavedMCRA \~English saved data struct for MCRA
 \~Japanese MCRAۑl\
 \param pSavedMCRA \~English saved data struct for MCRA(previous frame)
 \~Japanese Ot[MCRAۑl
 \param prmMCRA \~English parameter struct for MCRA
 \~Japanese MCRAp[^
 \param pSavedLeak \~English saved data struct for Leak noise
 \~Japanese RmCYۑl
 \param prmLeak \~English parameter struct for Leak noise
 \~Japanese RmCYp[^
 \param pSavedRev \~English saved data struct for Reverbaration
 \~Japanese cۑl
 \param pSavedRevtmp \~English saved data struct for Reverbration(previous frame)
 \~Japanese Ot[cۑl
 \param prmRev \~English parameter struct for Reverbraion
 \~Japanese cp[^
 \param pSavedSN \~English saved data struct for SN ratio
 \~Japanese SN䐄ۑl
 \param prmSN \~English parameter struct for SN ratio
 \~Japanese SN䐄p[^
 \param pSavedVoiceP \~English saved data struct for Speech probability
 \~Japanese ݊mۑl
 \param prmVoiceP \~English parameter struct for Speech probability
 \~Japanese ݊mp[^
 \param PrmGain \~English parameter optimum gain
 \~Japanese œKQCp[^
 \param pSavedLamda \~English saved data estimated back ground noise
 \~Japanese wimCYf[^
 */

int PF_ExecutePostFilter(MICARY_ComplexSpecArray csInputSpec , MICARY_ComplexSpecArray *pcsOutputSpec ,
							MICARY_SavedMCRA *pSavedMCRA ,MICARY_SavedMCRA *pSavedMCRAtmp ,MICARY_PrmMCRA prmMCRA ,
							MICARY_FloatSpecArray *pSavedLeak ,MICARY_PrmLeakNoise prmLeak ,
							MICARY_SavedEstRev *pSavedRev ,MICARY_SavedEstRev *pSavedRevtmp ,
							MICARY_PrmRevNoise prmRev , 
							MICARY_SavedEstSN *pSavedSN , MICARY_PrmEstSN prmSN,
							MICARY_SavedVoiceP *pSavedVoiceP , MICARY_PrmVoiceP prmVoiceP ,
			 /*MICARY_PrmGain PrmGain ,*/ MICARY_FloatSpecArray *pSavedLamda) ;

/**
 \brief \~Japanese MCRAwimCY̏(1:I, 0:G[)
 \param csInputSpec \~English Input spectrum
 \~Japanese ̓XyNg
 \param pfsOutputSpec \~English Output Noise spectrum
 \~Japanese o̓mCYXyNg
 \param ha_fNoiseCoef \~English Noise compensation value 
 \~Japanese mCY␳p[^
*/

int PF_InitMCRA(MICARY_ComplexSpecArray csInputSpec ,
				MICARY_FloatSpecArray *pfsOutputSpec ,HA_Float ha_fNoiseCoef) ;

// ---------------------------------------------------------------------
/**
 \brief \~English It calculate power spectrum .
 \brief \~Japanese p[XyNg̍쐬(1:I, 0:G[)
 \param pfsPowerSpec \English Power spectrum array struct pointer
 \~Japanese p[XyNgAC|C^
 \param csSpec \English Spectrum array struct pointer
 \~Japanese XyNgAC|C^
*/
int MakePowerSpec(MICARY_FloatSpecArray * pfsPowerSpec ,
					MICARY_ComplexSpecArray csSpec ) ;

#ifdef __cplusplus
}
#endif
// ----------------------------------------------------------------------------
#endif	// PF_EXEC_h_defined
