/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "BF.h"
#include "BF_Matrix.h"

namespace BF
{
    Location::Location(double _x, double _y, double _z, int _id) : x(_x), y(_y), z(_z), id(_id)
    {
    }

    Location::Location(const Location &src)
    {
		this->x = src.x;
		this->y = src.y;
		this->z = src.z;
		this->id = src.id;
    }
    
	Location &
	Location::operator = (const Location &src)
	{
		this->x = src.x;
		this->y = src.y;
		this->z = src.z;
		this->id = src.id;
		return *this;
	}

	double
	Location::euclidDistance(Location target)
	{
		double tmp = pow(this->x - target.x, 2.0)
					+ pow(this->y - target.y, 2.0)
					+ pow(this->z - target.z, 2.0);
		tmp = sqrt(tmp);
		return (tmp);
	}

    MicLocation::MicLocation(double _x, double _y, double _z, int _id) : Location(_x, _y, _z, _id)
    {
    }

    SourceLocation::SourceLocation(double _x, double _y, double _z, int _id, int _sourceID, bool _isSignal) : Location(_x, _y, _z, _id), sourceID(_sourceID), isSignal(_isSignal)
    {
    }

	SourceLocation &
	SourceLocation::operator = (const SourceLocation &src)
	{
		this->x = src.x;
		this->y = src.y;
		this->z = src.z;
		this->id = src.id;
		this->sourceID = src.sourceID;
		this->isSignal = src.isSignal;
		return *this;
	}

}
