/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "BF_ML.h"
#include <stdio.h>
#include <iostream>

namespace BF
{
#if 0
	CML::CML(int _numFreq, int _lowerfreq, int _upperfreq, int _samplingRate, double _soundSpeed, double _regFactor, double _epsValue) :
		BF(_numFreq, _lowerfreq, _upperfreq, _samplingRate, _soundSpeed, UtID), 
		regFactor(_regFactor), epsValue(_epsValue)
#endif
	CML::CML(int _numFreq, int _lowerfreq, int _upperfreq, int _samplingRate, double _soundSpeed, double _regFactor) :
		BF(_numFreq, _lowerfreq, _upperfreq, _samplingRate, _soundSpeed, UtID), 
		regFactor(_regFactor)
	{
	}

	CML::~CML()
	{
	}

	void
	CML::updateSources(const std::vector<SourceLocation> &sourcePositions)
	{
/*== source from MLBeam.m
	%----------------------------------
	%  Covariance
	%----------------------------------
	wnd = hamming(NFFT)';

	NFrame = floor((BlockLength-(FrameLength-FrameShift))/FrameShift);
	NBlock = floor(NData/BlockLength);

	CC = zeros(NFreq,M,M);

	BlockOrigin = BlockLength*(NoiseBlock-1);
	ZB = zeros(NFreq,M,NFrame);
	for frame = 1:NFrame
		t1 = BlockOrigin + FrameShift*(frame-1)+1;
		t2 = BlockOrigin + FrameShift*(frame-1)+FrameLength;
		for m=1:M
			Z = fft( z(m,t1:t2).*wnd, NFFT );
			ZB(:,m,frame) = reshape(Z(1:NFreq),NFreq,1,1);
		end
	end

	for freq=1:NFreq
		ZZ = reshape(ZB(freq,:,:),M,NFrame);
		KK(freq,:,:) = reshape(ZZ*ZZ',1,M,M);
	end

	%----------------------------
	%  Target loop
	%----------------------------

	for tl = 1:NTargetLoc
		target = TargetLocation(tl);
		%----------------------------
		%  Beamformer Design
		%----------------------------
		for k=1:NFFT/2+1
			a = reshape(AA(k,:,target),M,1);
			a = a/norm(a);
			K = reshape(KK(k,:,:),M,M);
			K = K + norm(K,'fro')*RegFactor*eye(M);
			invK = inv(K);
			WW(:,k) = invK*a/(a'*invK*a);
		end

	end

    for k=1:NFFT/2+1
        W = reshape(WW(:,k),1,M);
        H = exp(j*(k-1)*pi);						% Shift by NFFT/2;
        for m=1:M
            FF(k,m) = H*conj(W(m));                 % conj for y = w'x in freq. domain
        end
    end
    
    for k=NFFT/2+2:NFFT
        for m=1:M
            FF(k,m) = conj(FF(NFFT-k+2,m));
        end
    end
    
    for m=1:M
        F=reshape(FF(:,m),1,NFFT);
        ff(m,:) =  real( ifft(F,NFFT) );
    end
*/

		const int numSrc = sourcePositions.size();

		/*== 伝達関数を再計算する ==*/
		std::vector<Matrix> newHs;
		this->BF::getTransformFunctions(sourcePositions, newHs);

		/*== 分離行列を計算する ==*/
		std::vector<Matrix> newGs;
		for (int i = 0; i < this->numFreq; i++) {
			/* 伝達関数 */
			const Matrix &A = newHs[i];
			int numMic = A.getRowCount();

			/* 対象周波数のみ計算 */
	    	if (i == 0 || i == this->numFreq-1 || i < lowerfreq || i > upperfreq) {
				newGs.push_back(Matrix::O(numSrc, numMic));
				continue;
			}

			/* 相関行列 */
			Matrix K = this->covarianceMatrices[i];
			///K = K.conjugated();
			K = K + K.frobeniusNorm() * this->regFactor * Matrix::I(numMic);
			Matrix Kinv = K.pseudoInverse(); // Kの逆行列

			/* ML Beamformer */
			Matrix G(numSrc, numMic);
			for (int j = 0; j < numSrc; j++) {
				/* (K(-1) * Tf) / (Tf' * K(-1) * Tf) */

				Matrix Tf = A.getColumn(j);
				///Tf = Tf.conjugated();
				Matrix U, S, V;
				Tf.singularValueDecomposition(U, S, V);
				Complex Hnorm2 = S.getMaximumValue();
				/*if (abs(Hnorm2) < epsValue) { Hnorm2 = epsValue; }*/
//std::cout << "[" << i << "," << j << "]" << Hnorm2 << std::endl;
				if (Hnorm2 == 0.0)
					Tf = Tf * 0.0;
				else
					Tf = Tf / Hnorm2;
				Matrix Tft = Tf.transposed(); // Tfの複素共役転置

				//Matrix KinvTf = Kinv * Tf;
				//Matrix TftKinvTf = Tft * Kinv * Tf;

				Matrix TftKinv = Tft * Kinv;
				Matrix TftKinvTf = TftKinv * Tf;

				Complex sclrTftKinvTf = TftKinvTf.scalar();	// 1x1 の行列をスカラー値に変換
				/*if (abs(sclrTftKinvTf) < epsValue) { sclrTftKinvTf = epsValue; }*/

				Matrix W; // W [rows:numMic, cols:1]
				if (sclrTftKinvTf == 0.0)
					//W = KinvTf * 0.0;
					W = TftKinv * 0.0;
				else
					//W = KinvTf * (1.0 / sclrTftKinvTf);
					W = TftKinv / sclrTftKinvTf;

				//Complex H = exp(Complex(0.0, 1.0) * (double)i * M_PI);
 				//for (int k = 0; k < numMic; k++) {
 				//	Complex cW = W.getElementAt(k, 0);
 				//	G.setElementAt(j, k, H * std::conj(cW));
 				//}		
				G.setRow(j, W);
			}
			newGs.push_back(G);
		}

		/*== 結果の格納 ==*/
		this->sourcePositions = sourcePositions;
		newHs.swap(this->transformFunctions);
		newGs.swap(this->separationMatrices);
	}

	void
	CML::adaptSeparationMatrices()
	{
		/* do nothing */
	}
}
