/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include "BF.h"
#include "BF_Matrix.h"
#include <iostream>
#include <string.h>

namespace BF
{
    template<typename T>
    double distance(const T&a, const T&b)
    {
	return sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y) + (a.z - b.z) * (a.z - b.z));
    }

    int
    Database::findSourceIndex(const SourceLocation &sourceLocation) 
    {
		for(int i = 0; i < this->sourceLocations.size(); i++){
		    const SourceLocation &s = this->sourceLocations[i];
	    	if (sourceLocation.id == s.id){
				return i;
			}
		}
		// return (0);
		return (-1);
    }   
    int
    Database::findNearestSourceIndex(const SourceLocation &sourceLocation)
    {
	double mind = 0;
	int    mini = 0;
	for(int i = 0; i < this->sourceLocations.size(); i++){
	    const SourceLocation &s = this->sourceLocations[i];
	    double d = distance(s, sourceLocation);
	    if(i == 0 || d < mind){
		mind = d;
		mini = i;
	    }
	}

	return mini;
    }
    int
    Database::findNearestSourceIndex(const MicLocation &micLocation)
    {
	double mind = DBL_MAX;
	int    mini = -1;
	for(int i = 0; i < this->micLocations.size(); i++){
	    const MicLocation &m = this->micLocations[i];
	    double d = distance(m, micLocation);
	    if(i == 0 || d < mind){
		mind = d;
		mini = i;
	    }
	}

	return mini;
    }    
}
