/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <iostream> 
#include "CParticleFilter.h"

//-----------------------------------------------------------
CParticleFilter::CParticleFilter()
{
	//コンストラクタ
	
	pg_set.pg = NULL;
	pg_set.pg_num = 0;
	pg_set.pg_alive_num = 0;
	i_ob_num_IRMA = 0;
	i_ob_num_REMA = 0;
	fp_file = NULL;

	i_FrameCount = 0; //フレーム番号初期化
}
//-----------------------------------------------------------
CParticleFilter::~CParticleFilter()
{
	//デストラクタ
	int i;
	for(i = 0; i < pg_set.pg_num; i++)
	{
		free(pg_set.pg[i].state);
		pg_set.pg[i].state = NULL;
		free(pg_set.pg[i].weight);
		pg_set.pg[i].weight = NULL;
		free(pg_set.pg[i].h);
		pg_set.pg[i].h = NULL;
		
	}

	if(pg_set.pg){
		free(pg_set.pg);
		pg_set.pg = NULL;
	}
}
//-----------------------------------------------------------
void CParticleFilter::ExecuteParticleFilter(const int flag)
{


	int Pnum;	//Pnum=Particle総数, i_FrameCount=フレーム番号
	int j;
	int max_source;


	ob_set.ob = NULL;
	ob_set.ob_FrameID = 0;
	ob_set.ob_num = 0;

	r_set.r_ob = NULL;
	r_set.ob_FrameID = 0;
	r_set.ob_num = 0;

	//来た入力データをob_set(またはr_set)にセットする(1027追加)
	this->SetData(flag);

	Pnum = s_param.m_param_particle_num;
	max_source = s_param.m_param_max_num_of_source;  
	
	//----------------------実際の処理開始----------------------
	check_source(flag, Pnum, &pg_set, &ob_set, &r_set, s_param);		//パーティクルの初期化

	if(pg_set.pg_num > 0)
	{
			//std::cout << "pg_set.pg_num bimportance_sampling = " << pg_set.pg_num <<std::endl;
			//重要度のサンプリング
			//tfindexの更新はここ
			importance_sampling(flag, Pnum, &pg_set, &ob_set, &r_set, s_param); 

			//std::cout << "pg_set.pg_num aimportance_sampling = " << pg_set.pg_num <<std::endl;

			if(ob_set.ob == NULL)
			{				
				state_output(flag, Pnum, &pg_set, r_set.ob_FrameID, s_param); //Particleの現在の状態を出力する
			}
			else
			{
				state_output(flag, Pnum, &pg_set, ob_set.ob_FrameID, s_param); //Particleの現在の状態を出力する
			}

			//出力データをvectorにセットする
			output.clear(); 

			for(j = 0; j < pg_set.pg_num; j++)
			{
				//s_out.id = pg_set.pg[j].index + 1;	//IDは1から。
				s_out.id = pg_set.pg[j].index;		//IDは0スタート　2016.9.30
				s_out.azimuth = pg_set.pg[j].h->theta;
				s_out.elevation = pg_set.pg[j].h->phi;

				s_out.x[0] = pg_set.pg[j].h->x;
				s_out.x[1] = pg_set.pg[j].h->y;
				
				// Changed by Morio べた書き解除
				//pg_set.pg[j].h->z = 0.287361;	//音源の高さ、べた書き
				s_out.x[2] =pg_set.pg[j].h->z;

				// tfindex 2016.9.30
				s_out.tfindex[0] = pg_set.pg[j].tfindex[0];

				if(pg_set.pg[j].pg_status == 0)
				{
					output.push_back(s_out);
				}

			}
			selection(Pnum, &pg_set, s_param);  //Particleの除去、増殖
			//i_FrameCount++; //処理が正常にまわったらカウンタを増やす
	}

	i_FrameCount++; //カウンタを増やす
	if(r_set.r_ob )
	{
		delete r_set.r_ob;
		r_set.r_ob = NULL;
	}

	if(ob_set.ob)
	{
		delete ob_set.ob;
		ob_set.ob = NULL;
	}
}
//-----------------------------------------------------------
//入力データ(観測値)をセットする

void CParticleFilter:: SetData(const int flag)
{

	int j;
	double rob_theta, rob_phi;

	//入力された観測値データで場合わけ。
	if(flag == ROBOT_DATA)
	{
		r_set.ob_num = i_ob_num_REMA;
 		r_set.ob_FrameID = i_FrameCount;

		if(i_FrameCount == 0){
			pg_set.pg = NULL;
		}

	//領域確保
		if(r_set.ob_num > 0)
		{
			r_set.r_ob = new robot_observation_type[r_set.ob_num];
		}
		
		//来た入力データをr_setにセット
			for(j = 0; j < r_set.ob_num; j++)
			{
				//ロボットの位置べた書き（追々は，NodeへのInputデータより）
				// Changed by Morio べた書き解除
				

				rob_theta = 0.0;	//頭部の角度?
				rob_phi = 0.0;	//頭部の角度?
			
				//r_set.r_ob[j].x = input_r[j].x[0];
				//r_set.r_ob[j].y = input_r[j].x[1];
				//r_set.r_ob[j].z = input_r[j].x[2];
				r_set.r_ob[j].r_theta = rob_theta;
				r_set.r_ob[j].r_phi = rob_phi;
				r_set.r_ob[j].theta = input_r[j].azimuth;
				r_set.r_ob[j].phi = input_r[j].elevation;
				r_set.r_ob[j].likelihood = input_r[j].power;

				// tfindex 2016.9.30
				r_set.r_ob[j].tfindex[0] = input_r[j].tfindex[0];
			}
	}
	
}
//-----------------------------------------------------------
//パラメータのデフォルト設定（必要があれば）
void  CParticleFilter::initial_param(const int flag)
{
	if(flag == ROBOT_DATA){
		s_param.m_param_particle_num = 1000;
		s_param.m_param_max_num_of_source = 1;
		s_param.m_param_th_another_source = 0.00001;	//最大尤度がこれ以下であれば　PGと観測値との対応付けを終了する
		s_param.m_param_th_ignore_source = 0.00001; 	//これ以下の尤度であれば新音源とみなす
		s_param.m_param_th_remove_source = 100;		//グループが削除されるまでのフレーム数
		s_param.m_param_th_output_range = 20.0;		//近傍の粒子だと認識するための範囲．robの場合は角度[°]
		s_param.m_param_likelihood_sigma = 0.0;		//REMAでは使わないので，定数0.0に。(0316)
		s_param.m_param_likelihood_sigma_robot = 200.0;
		s_param.m_param_likelihood_robot_weight = 1.0;
		//s_param.m_param_th_random_walk = 2.0;		//[m/s]
		s_param.m_param_state_update_sigma =10.0;	//内部状態の遷移の際に加えられるランダムの分散具合　小さくしすぎると観測値との差が大きくなりすぎて意味の無いことに
		s_param.m_param_sum_weight = 0.4;	
		s_param.m_param_room_length = 5.5;			//室内の大きさ ->未使用
		s_param.m_param_room_width = 6.5;			//室内の大きさ ->未使用
		s_param.m_param_rob_pow = 30.0;				//powerの閾値 -> 未使用
		s_param.m_param_room_pow = -78.0;			//powerの閾値 -> 未使用
		s_param.m_param_history = 1;				//logを残す:1,残さない:0
	}

}
//-----------------------------------------------------------
//入力されたパラメータをパラメータ構造体に取り直す
void  CParticleFilter::set_param(const int param_particle_num, 
					const int param_max_num_of_source,
					const double param_th_another_source,
					const double param_th_ignore_source,
					const int param_th_remove_source,
					const double param_th_output_range, 
					const double param_likelihood_sigma,
					const double param_likelihood_sigma_robot,
					const double param_likelihood_robot_weight,
					const double param_th_random_walk,
					const double param_state_update_sigma,
					const double param_sum_weight,		
					const double param_rob_pow,
					const double param_room_length, 
					const double param_room_width, 
					const double param_room_pow,
					const bool param_history)
{

	if(param_history == false){
		s_param.m_param_history = 0;	//harkmwからboolでくるので，C言語でも使えるようにintに変換
	}
	else{
		s_param.m_param_history = 1;
	}

	s_param.m_param_particle_num = param_particle_num;
	s_param.m_param_max_num_of_source = param_max_num_of_source;
	s_param.m_param_th_another_source = param_th_another_source;
	s_param.m_param_th_ignore_source = param_th_ignore_source;
	s_param.m_param_th_remove_source = param_th_remove_source;
	s_param.m_param_th_output_range = param_th_output_range;
	s_param.m_param_likelihood_sigma = param_likelihood_sigma,
	s_param.m_param_likelihood_sigma_robot = param_likelihood_sigma_robot;
	s_param.m_param_likelihood_robot_weight = param_likelihood_robot_weight;
	//s_param.m_param_th_random_walk = param_th_random_walk;
	s_param.m_param_state_update_sigma = param_state_update_sigma;
	s_param.m_param_sum_weight = param_sum_weight;
	s_param.m_param_rob_pow = param_rob_pow;
	s_param.m_param_room_length = param_room_length;
	s_param.m_param_room_pow = param_room_pow;
	s_param.m_param_room_width = param_room_width;

	//printf("likelihood_sigma = %f\n",s_param.m_param_likelihood_sigma);
}
//-----------------------------------------------------------
//パラメータのファイル出力(必要があれば)
void CParticleFilter::print_param()
{
	FILE *fp_set_param;	

	fp_set_param = fopen("print_param.txt","w");

	fprintf(fp_set_param,"Total_particle = %d;\nMax_num_of_source = %d;\nAnother_source = %f;\nIgnore_source = %f;\nRemove_source = %d;\nOutput_range = %f;\nLikelihood_sigma = %f;\nLikelihood_sigma_robot = %f;\nLikelihood_robot_weight = %f;\nRandom_walk = %f;\nState_update_sigma = %f;\nSum_weight = %f;\nRob_pow = %f;\nRoom_length = %f;\nRoom_width = %f;\nRoom_pow = %f;\n",
		s_param.m_param_particle_num,
		s_param.m_param_max_num_of_source,
		s_param.m_param_th_another_source,
		s_param.m_param_th_ignore_source,
		s_param.m_param_th_remove_source,
		s_param.m_param_th_output_range,
		0.0f,//s_param.m_param_likelihood_sigma,
		s_param.m_param_likelihood_sigma_robot,
		s_param.m_param_likelihood_robot_weight,
		0.0f,//s_param.m_param_th_random_walk,
		s_param.m_param_state_update_sigma,
		s_param.m_param_sum_weight,
		s_param.m_param_rob_pow,
		s_param.m_param_room_length,
		s_param.m_param_room_width,
		s_param.m_param_room_pow);

	fclose(fp_set_param);
}
