/*
 * Copyright (c) 2008 - 2020 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 * 
 * Licensed under the HARK license, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * https://www.hark.jp/notice/HARK_License_Agreement.pdf
 */

#include <iostream>
#include <BufferedNode.h>
#include <Buffer.h>
#include <Matrix.h>
#include <Vector.h>
#include <Stream.h>
#include <Map.h>
#include <Time.h>

using namespace HARK;

class DataLoggerOld;

DECLARE_NODE(DataLoggerOld)
/*Node
 *
 * @name DataLoggerOld
 * @category HARK:MISC
 * @description Log the input to stdout.
 *
 * @input_name INPUT
 * @input_type any
 * @input_description The input is what you want to log.
 *
 * @output_name OUTPUT
 * @output_type any
 * @output_description The output is the same as the input.
 *
 * @parameter_name LABEL
 * @parameter_type string
 * @parameter_description add a label to logged data
 *
 END*/

class DataLoggerOld: public BufferedNode {
    int inputID;
    int outputID;

    std::string label;

public:
    DataLoggerOld(std::string nodeName, ParameterSet params) :
            BufferedNode(nodeName, params) {
        inputID = addInput("INPUT");
        outputID = addOutput("OUTPUT");

        if(parameters.exist("LABEL")){
            label = object_cast<std::string>(parameters.get("LABEL"));
        }

        inOrder = true;
    }

    void calculate(int output_id, int count, Buffer &out) {
        ObjectRef input(getInput(inputID, count));
        out[count] = input;

        std::cout << label << count << " ";
        if(typeid(*input) == typeid(Map<int, ObjectRef>)){
            RCPtr<Map<int, ObjectRef> > in = input;
            Map<int, ObjectRef>::iterator it = in->begin();
            for(; it != in->end(); ++it){
                const int id = it->first;
                if(typeid(*(it->second)) == typeid(Vector<float>)){
                    RCPtr<Vector<float> > vec = it->second;
                    std::cout << id << " ";
                    for(int i = 0; i < vec->size(); i++){
                        std::cout << (*vec)[i] << " ";
                    }
                }
                else if(typeid(*(it->second)) == typeid(Vector<std::complex<float> >)){
                    RCPtr<Vector<std::complex<float> > > vec = it->second;
                    std::cout << id << " ";
                    for(int i = 0; i < vec->size(); i++){
                        std::cout << (*vec)[i] << " ";
                    }
                }
            }
        }
        else if(typeid(*input) == typeid(Map<int, float>)){
            RCPtr<Map<int, float> > in = input;
            Map<int, float>::iterator it = in->begin();
            for(; it != in->end(); ++it){
                const int id = it->first;
                std::cout << id << " " << it->second << " ";
            }
        }
        else if(typeid(*input) == typeid(Map<int, int>)){
            RCPtr<Map<int, int> > in = input;
            Map<int, int>::iterator it = in->begin();
            for(; it != in->end(); ++it){
                const int id = it->first;
                std::cout << id << " " << it->second << " ";
            }
        }
        else if(typeid(*input) == typeid(Time)){
            RCPtr<Time> in = input;
            std::cout << _TimeLog(in->time);
        }
        std::cout << std::endl;
    }

    IN_ORDER_NODE_SPEEDUP(DataLoggerOld);
};
